using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Tile : MonoBehaviour 
{
	public const int TILE_SIZE = 32;
	
	public RagePixelSprite sprite;
	
	public Tile north;
	public Tile east;
	public Tile south;
	public Tile west;
	
	public Citizen citizen;
	public OOD ood;
	
	public List<DAttribute> attributes;
	
	public DAttribute tileattr;
	
	public enum TileType { Grass, Sand, Mountain, Lake };
	public TileType type = TileType.Grass;
	public string spriteName;
	public string description;
	
	public Transform citizenOffset;
	Transform textOffset;
	
	public bool processedUpdate;
	
	protected virtual void Start () 
	{
		sprite = GetComponent<RagePixelSprite>();
		
		east = Tile.RaycastTile(transform.position + new Vector3(TILE_SIZE, 0, 0));
		west = Tile.RaycastTile(transform.position + new Vector3(-TILE_SIZE, 0, 0));
		north = Tile.RaycastTile(transform.position + new Vector3(0, TILE_SIZE, 0));
		south = Tile.RaycastTile(transform.position + new Vector3(0, -TILE_SIZE, 0));
		
		//citizenOffset = transform.FindChild("CitizenOffset");
		textOffset = transform.FindChild("TextOffset");
		
		attributes = new List<DAttribute>();
		tileattr = new DAttribute();
		attributes.Add(tileattr);
		
		SetTile(type);
	}
	
	static string greenName = "grass field";
	static string yellowName = "desert";
	static string blueName = "lake";
	static string redName = "mountain";
	
	public void SetTile(TileType newType)
	{
		type = newType;
		
		switch (type)
		{
		case TileType.Grass:
			tileattr.name = greenName;
			tileattr.color = Citizen.green;
			tileattr.type = DAttribute.AttrType.Place;
			spriteName = "Grass";
			description = "Grass Field";
			break;
		case TileType.Sand:
			tileattr.name = yellowName;
			tileattr.color = Citizen.yellow;
			tileattr.type = DAttribute.AttrType.Place;
			spriteName = "Sand";
			description = "Desert";
			break;
		case TileType.Mountain:
			tileattr.name = redName;
			tileattr.color = Citizen.red;
			tileattr.type = DAttribute.AttrType.Place;
			spriteName = "Mountain";
			description = "Mountain";
			break;
		case TileType.Lake:
			tileattr.name = blueName;
			tileattr.color = Citizen.blue;
			tileattr.type = DAttribute.AttrType.Place;
			spriteName = "Lake";
			description = "Lake";
			break;
		}
		
		sprite.SetSprite(spriteName);
	}
	
	public static string RandomTileName()
	{
		switch (Random.Range(0, 4))
		{
		case 0:
			return greenName;
		case 1:
			return yellowName;
		case 2:
			return redName;
		case 3:
			return blueName;
		}
		
		return greenName; // never happen
	}
	
	void Update () 
	{
		processedUpdate = false;
	}
	
	
	public static Tile RaycastTile(Vector3 vec)
	{
		return RaycastTile(vec.x, vec.y);
	}
	
	public static Tile RaycastTile(float x, float y)
	{
		RaycastHit hit;
		
		if (Physics.Raycast(new Vector3(x, y, -10), Vector3.forward, out hit, 20, ~LayerMask.NameToLayer("Tiles")))
		{
			return hit.transform.gameObject.GetComponent<Tile>();
		}
		
		return null;
	}
	
	public void OnPressDown()
	{
		Debug.Log("down on tile: " + transform.position);
		
		Level.Instance.SelectTile(this);
	}
	
	public void OnPressUp()
	{
		Debug.Log("pressed up: " + transform.position);
	}
	
	public void OnHover()
	{
		Level.Instance.SelectTile(this);
	}
	
	public void DroppedObject(GameObject dropped)
	{
		if (citizen)
		{
			// one citizen per tile
			DragObject.RejectDrop(dropped);
		}
		else
		{
			citizen = dropped.GetComponent<Citizen>();
			
			if (citizen == null)
				DragObject.RejectDrop(dropped);
			else
			{
				citizen.collider.enabled = false; // want to hit the tile instead of citizen
				citizen.transform.parent = transform;
				Destroy(citizen.GetComponent<DragObject>());
				
				// stamp
				Vector3 temp = citizen.transform.position;
				citizen.transform.position = citizenOffset.position;
				Level.Instance.SelectTile(this);
				citizen.transform.position = temp;
				
				iTween.MoveTo(citizen.gameObject, citizenOffset.position, 0.2f);
				
				Raft.Instance.CheckCapacity();
				
				string n = citizen.citizenName;
				
				if (n.Length > 5)
					n = n.Substring(0, 5) + ".";
				
				if (citizen.citizenName == "Samantha")
					n = "Sam.";
				
				InfoHUD.Instance.AddText(textOffset, n);
				
				CleanUpdateDesires(true);
			}
		}
	}
	
	public void CleanUpdateDesires()
	{
		CleanUpdateDesires(false);
	}
	
	public void CleanUpdateDesires(bool force)
	{
		if (!citizen) return;
		if (processedUpdate) return;
		
		processedUpdate = true; // before applying attributes
		
		//Debug.Log("--------------------------------------------------------");
		//Debug.Log("CleanUpdateDesires: " + citizen.citizenName);
		//Debug.Log("--------------------------------------------------------");
		
		citizen.DesireUpdateBegin();
		
		ApplyTileAttributes(this);
		ApplyTileAttributes(north);
		ApplyTileAttributes(east);
		ApplyTileAttributes(south);
		ApplyTileAttributes(west);
		
		// recursively update (very rarely will this be needed and must always happen when an object is placed)
		if (citizen.DesiresChanged() || force)
		{	
			if (north) north.CleanUpdateDesires();
			if (east) east.CleanUpdateDesires();
			if (south) south.CleanUpdateDesires();
			if (west) west.CleanUpdateDesires();
		}
	}
	
	public void ApplyTileAttributes(Tile t)
	{
		if (t == null) return;
		
		// tile attributes
		if (t.citizen && citizen == t.citizen)
			citizen.UpdateDesires(t.attributes, true); // update on own tile
		else
			citizen.UpdateDesires(t.attributes, false);
		
		// citizen of that tile (excluding you)
		if (t.citizen && citizen != t.citizen)
		{
			//Debug.Log("Applied citizen update '" + t.citizen.citizenName + "' to " + t.transform.position); 
			citizen.UpdateDesires(t.citizen.attributes, false);
		}
	}
}
